#ifndef __VIEWCELLSTREE
#define __VIEWCELLSTREE

#include <vector>
#include "Pvs.h"
#include "AxisAlignedBox3.h"
#include "common.h"

	
namespace CHCDemoEngine
{

/** basic view cell structure
*/
struct ViewCell 
{
public:

	ViewCell(): mPvs() {}

	int GetId() const { return mId; }

	void SetId(int id) { mId = id; }

	void SetBox(const AxisAlignedBox3 &box) { mBox = box; }


	// the pvs associated with the viewcell
	Pvs mPvs;

protected:

	// id of this viewcell
	int mId;
	// bounding box of the viewcell
	AxisAlignedBox3 mBox;
};


struct ViewCellsTreeNode
{
	ViewCellsTreeNode(): mAxis(-1), mViewCell(NULL) {}

	bool IsLeaf() { return mViewCell != NULL; }


	///////////////////////

	union 
	{
		// split position - 4B
		float mPosition;
		// the viewcell associated with this node (NULL for non terminal nodes)
		ViewCell *mViewCell; 
	};

	/// children of the node
	ViewCellsTreeNode *mBack;
	ViewCellsTreeNode *mFront;

	/// split mAxis 
	int mAxis;
};


class ViewCellsTree
{
	friend class VisibilitySolutionLoader;

public:
	
	ViewCellsTree(): mRoot(NULL) {}

	void SubdivideNode(ViewCellsTreeNode *node,
                       const AxisAlignedBox3 &box,
					   int depth);


	int	CastLineSegment(const Vector3 &origin,
                        const Vector3 &termination,
						ViewCell **viewcells);

	ViewCell *GetViewCell(const Vector3 &point) const;

	bool LoadFromFile(const std::string &filename);


protected:

	bool _LoadFromFile(FILE *file);


	////////////////////////

	int mMaxDepth;
	AxisAlignedBox3 mBox;
	ViewCellsTreeNode *mRoot;
};

}

#endif
